package App::Prove::Plugin::retty;
use strict;
use warnings;
use utf8;
use parent qw/App::Prove::Plugin::Pretty/;

1;
__END__

=head1 NAME

App::Prove::Plugin::retty - -Pretty

=head1 SYNOPSIS

    % prove -Pretty

=head1 DESCRIPTION

This is a alias module for App::Prove::Plugin::Pretty.

Using C<< -Pretty >> is bit pretty.
